@@ -1,6 +1,6 @@
 package Dancer2::Plugin::Auth::Extensible;
 
-our $VERSION = '0.703';
+our $VERSION = '0.704';
 
 use strict;
 use warnings;
@@ -11,7 +11,7 @@ use Dancer2::Template::Tiny;
 use File::Share qw(dist_dir);
 use List::Util qw(first);
 use Module::Runtime qw(use_module);
-use Scalar::Util;
+use Scalar::Util qw(blessed);
 use Session::Token;
 use Try::Tiny;
 use URI::Escape;
@@ -669,7 +669,8 @@ sub password_reset_send {
             # separate provider function to get an email address, which seems
             # an overkill. Providers can make the email key configurable if
             # need be
-            my %options = ( code => $code, email => $user->{email} );
+            my $email = blessed $user ? $user->email : $user->{email};
+            my %options = ( code => $code, email => $email );
 
             no strict 'refs';
             $result++
@@ -938,7 +939,7 @@ sub _build_wrapper {
         my $view            = $plugin->denied_page;
         my $template_engine = $plugin->app->template_engine;
         my $path            = $template_engine->view_pathname($view);
-        if ( !-f $path ) {
+        if ( !$template_engine->pathname_exists($path) ) {
             $plugin->app->log(
                 debug => "app has no denied_page template defined" );
             $options->{content} = $plugin->_render_template('login_denied.tt');
@@ -990,18 +991,7 @@ sub _check_for_login {
                 '__auth_extensible_params' => \%{ $request->params } );
         }
 
-        # If app has its own login page view then use it
-        # otherwise render our internal one and pass that to 'template'.
-        my ( $view, $options ) = ( $plugin->login_template, {} );
-        my $template_engine = $plugin->app->template_engine;
-        my $path            = $template_engine->view_pathname($view);
-        if ( !-f $path ) {
-            $plugin->app->log( debug => "app has no login template defined" );
-            $options->{content} =
-              $plugin->_render_template( 'transparent_login.tt', $tokens );
-            undef $view;
-        }
-        return $plugin->app->template( $view, $tokens, $options );
+        return $plugin->_render_login_page( 'transparent_login.tt', $tokens );
     }
 
     # old-fashioned redirect to login page with return_url set
@@ -1012,6 +1002,23 @@ sub _check_for_login {
     );
 }
 
+sub _render_login_page {
+    my ( $plugin, $default_template, $tokens ) = @_;
+
+    # If app has its own login page view then use it
+    # otherwise render our internal one and pass that to 'template'.
+    my ( $view, $options ) = ( $plugin->login_template, {} );
+    my $template_engine = $plugin->app->template_engine;
+    my $path            = $template_engine->view_pathname($view);
+    if ( !$template_engine->pathname_exists($path) ) {
+        $plugin->app->log( debug => "app has no login template defined" );
+        $options->{content} =
+          $plugin->_render_template( $default_template, $tokens );
+        undef $view;
+    }
+    return $plugin->app->template( $view, $tokens, $options );
+}
+
 sub _default_email_password_reset {
     my ( $plugin, %options ) = @_;
 
@@ -1071,17 +1078,7 @@ sub _default_login_page {
         return_url => uri_escape( $request->parameters->get('return_url') ),
     };
 
-    # If app has its own login page view then use it
-    # otherwise render our internal one and pass that to 'template'.
-    my ( $view, $options ) = ( $plugin->login_template, {} );
-    my $template_engine = $plugin->app->template_engine;
-    my $path            = $template_engine->view_pathname($view);
-    if ( !-f $path ) {
-        $plugin->app->log( debug => "app has no login template defined" );
-        $options->{content} = $plugin->_render_template( 'login.tt', $tokens );
-        undef $view;
-    }
-    return $plugin->app->template( $view, $tokens, $options );
+    return $plugin->_render_login_page( 'login.tt', $tokens );
 }
 
 sub _default_permission_denied_page {
